#include "Best.hpp"
#include "Problem.hpp"
#include "StringUtils.hpp"
#include "AtomicFileUtils.hpp"
#include "IsValid.hpp"
#include "CalcScore.hpp"
#include "ProblemCount.hpp"

using std::string;

Best::Best() :
    m_id(0),
    m_score(0)
{
}

Best::~Best()
{
}

namespace
{
    string getFileName(uint64_t id)
    {
        return strprintf("../solutions/%" PRIu64 "", id);
    }
}

void Best::load(uint64_t id)
{
    m_id = id;

    string msg;
    if (!parseProblem(id, &m_problem, &msg))
    {
        fprintf(stderr, "%s\n", msg.c_str());
        exit(1);
    }

    m_solutionData.clear();

    string fileName = getFileName(id);
    atomicReadFile(fileName, &m_solutionData);

    parse();
}

bool Best::update(const Solution& solution)
{
    uint64_t id = m_id;

    if (id < 1 || id > problemCount)
    {
        return false;
    }

    if (!isValid(m_problem, solution))
    {
        return false;
    }

    int64_t score = calcScore(m_problem, solution);

    while (true)
    {
        if (score <= m_score)
        {
            return false;
        }

        m_solution = solution;
        m_score = score;

        string solutionData;
        formatSolution(m_solution, &solutionData);

        string fileName = getFileName(id);

        if (atomicReplaceFile(fileName, &m_solutionData, solutionData))
        {
            m_solutionData.swap(solutionData);
            return true;
        }

        parse();
    }
}

void Best::parse()
{
    if (!parseSolution(m_solutionData, &m_solution) ||
        !isValid(m_problem, m_solution))
    {
        m_solution = Solution();
        m_score = 0;
    }
    else
    {
        m_score = calcScore(m_problem, m_solution);
    }
}
