#ifndef CALCSCORE_HPP
#define CALCSCORE_HPP

#include "Common.hpp"
#include "Problem.hpp"
#include "Checker.hpp"
#include <cmath>

inline int64_t calcScore(const Problem& problem,
                         const Solution& solution,
                         bool useCloseness)
{
    uint64_t attendeeCount = problem.attendees.size();
    uint64_t musicianCount = problem.musicians.size();

    int64_t score = 0;
    for (uint64_t musicianIdx = 0; musicianIdx < musicianCount; musicianIdx++)
    {
        Coord musicianCoord = solution.placements[musicianIdx];

        double volume = solution.volumes.empty() ? 1.0 : solution.volumes[musicianIdx];

        if (volume == 0.0) continue;

        double closeness = 1.0;
        if (useCloseness)
        {
            for (uint64_t otherMusicianIdx : problem.instruments[problem.musicians[musicianIdx]])
            {
                if (otherMusicianIdx == musicianIdx) continue;
                Coord otherMusicianCoord = solution.placements[otherMusicianIdx];
                double d2 = calcD2(musicianCoord, otherMusicianCoord);
                double d = std::sqrt(d2);
                closeness += 1.0 / d;
            }
        }

        for (uint64_t attendeeIdx = 0; attendeeIdx < attendeeCount; attendeeIdx++)
        {
            auto& attendee = problem.attendees[attendeeIdx];
            Coord attendeeCoord = attendee.coord;
            ObstructionChecker oc(attendeeCoord, musicianCoord);

            bool blocked = false;
            for (uint64_t testIdx = 0; testIdx < musicianCount; testIdx++)
            {
                if (testIdx == musicianIdx) continue;
                Coord testCoord = solution.placements[testIdx];
#if 0
                AttendeeChecker ac(musicianCoord, testCoord, 5.0);
                bool ocResult = oc.check(testCoord, 5.0);
                bool acResult = ac.check(attendeeCoord);
                if (ocResult != acResult)
                {
                    printf("musician: ocResult = %s, acResult = %s\n", ocResult ? "true" : "false", acResult ? "true" : "false");
                    exit(1);
                }
#endif
                if (oc.check(testCoord, 5.0))
                {
                    blocked = true;
                    break;
                }
            }

            if (blocked) continue;

            for (auto& pillar : problem.pillars)
            {
#if 0
                AttendeeChecker ac(musicianCoord, pillar.coord, pillar.radius);
                bool ocResult = oc.check(pillar.coord, pillar.radius);
                bool acResult = ac.check(attendeeCoord);
                if (ocResult != acResult)
                {
                    printf("pillar: ocResult = %s, acResult = %s\n", ocResult ? "true" : "false", acResult ? "true" : "false");
                    exit(1);
                }
#endif
                if (oc.check(pillar.coord, pillar.radius))
                {
                    blocked = true;
                    break;
                }
            }

            if (blocked) continue;

            double d2 = calcD2(attendeeCoord, musicianCoord);
            double taste = attendee.tastes[problem.musicians[musicianIdx]];

            double impact = std::ceil(1000000.0 * taste / d2);
            impact = std::ceil(volume * closeness * impact);
            score += (int64_t)impact;
        }
    }

    return score;
}

inline int64_t calcScore(const Problem& problem,
                         const Solution& solution)
{
    return calcScore(problem, solution, !problem.pillars.empty());
}

#endif
