#include "FormatUtils.hpp"
#include "StringUtils.hpp"

using std::string;

string formatWithCommas(uint64_t val)
{
    string str = strprintf("%" PRIu64 "", val);
    size_t size = str.size();
    size_t numCommas = (size - 1) / 3;
    str.resize(size + numCommas);
    size_t pos = size + numCommas;
    for (size_t i = 0; i < size; i++)
    {
        if (i > 0 && (i % 3 == 0))
        {
            pos--;
            str[pos] = ',';
        }
        pos--;
        str[pos] = str[size - 1 - i];
    }
    return str;
}

string formatWithCommas(int64_t val)
{
    string str;
    if (val < 0)
    {
        str = formatWithCommas(-(uint64_t)val);
        str.insert(str.begin(), '-');
    }
    else
    {
        str = formatWithCommas((uint64_t)val);
    }
    return str;
}

string formatSI(uint64_t val, uint64_t digits)
{
    uint64_t thousands = 0;
    uint64_t thousandsPower = 1;
    uint64_t intPart = val;
    while (intPart >= 1000)
    {
        thousands++;
        thousandsPower *= 1000;
        intPart /= 1000;
    }
    if (thousands == 0)
    {
        return strprintf("%" PRIu64 "", intPart);
    }
    uint64_t intDigits = (intPart < 10) ? 1 : (intPart < 100) ? 2 : 3;
    char prefix = " kMGTPE"[thousands];
    uint64_t fracDigits = thousands * 3;
    if (intDigits >= digits)
    {
        fracDigits = 0;
    }
    else if (fracDigits + intDigits > digits)
    {
        fracDigits = digits - intDigits;
    }
    if (fracDigits == 0)
    {
        return strprintf("%" PRIu64 "%c", intPart, prefix);
    }
    uint64_t fracPart = val % thousandsPower;
    uint64_t curFracDigits = thousands * 3;
    while (curFracDigits > fracDigits)
    {
        fracPart /= 10;
        curFracDigits--;
    }
    return strprintf("%" PRIu64 ".%0*" PRIu64 "%c", intPart, (int)fracDigits, fracPart, prefix);
}

string formatDurationS(uint64_t durationS)
{
    uint64_t s = durationS % 60;
    uint64_t m = (durationS / 60) % 60;
    uint64_t h = (durationS / (60 * 60));
    return strprintf("%02" PRIu64 ":%02" PRIu64 ":%02" PRIu64 "s", h, m, s);
}

string formatDurationMS(uint64_t durationMS)
{
    uint64_t ms = durationMS % 1000;
    uint64_t s = (durationMS / 1000) % 60;
    uint64_t m = (durationMS / (1000 * 60)) % 60;
    uint64_t h = (durationMS / (1000 * 60 * 60));
    return strprintf("%02" PRIu64 ":%02" PRIu64 ":%02" PRIu64 ".%03" PRIu64 "s", h, m, s, ms);
}
