#include "IsValid.hpp"
#include "Problem.hpp"

bool isValid(const Problem& problem,
             const Solution& solution)
{
    uint64_t musicianCount = problem.musicians.size();

    if (solution.placements.size() != musicianCount)
    {
        return false;
    }

    if (!solution.volumes.empty() && solution.volumes.size() != musicianCount)
    {
        return false;
    }

    double stageLowX = problem.stageX + 10.0;
    double stageHighX = problem.stageX + problem.stageWidth - 10.0;
    double stageLowY = problem.stageY + 10.0;
    double stageHighY = problem.stageY + problem.stageHeight - 10.0;

    for (auto& coord : solution.placements)
    {
        if (coord.x < stageLowX ||
            coord.x > stageHighX ||
            coord.y < stageLowY ||
            coord.y > stageHighY)
        {
            return false;
        }
    }

    for (uint64_t i = 0; i < musicianCount; i++)
    {
        Coord coordI = solution.placements[i];
        for (uint64_t j = i + 1; j < musicianCount; j++)
        {
            Coord coordJ = solution.placements[j];
            double d2 = calcD2(coordI, coordJ);
            if (d2 < 100.0)
            {
                return false;
            }
        }
    }

    for (double volume : solution.volumes)
    {
        if (volume < 0.0 || volume > 10.0)
        {
            return false;
        }
    }

    return true;
}
