#ifndef PROBLEM_HPP
#define PROBLEM_HPP

#include "Common.hpp"
#include "Coord.hpp"

class Attendee
{
public:
    Coord coord;
    std::vector<double> tastes;
};

class Pillar
{
public:
    Coord coord;
    double radius = 0.0;
};

class Problem
{
public:
    double roomWidth = 0.0;
    double roomHeight = 0.0;
    double stageWidth = 0.0;
    double stageHeight = 0.0;
    double stageX = 0.0;
    double stageY = 0.0;
    std::vector<uint64_t> musicians;
    std::vector<Attendee> attendees;
    std::vector<std::vector<uint64_t>> instruments;
    std::vector<Pillar> pillars;
};

class Solution
{
public:
    std::vector<Coord> placements;
    std::vector<double> volumes;
};

bool readProblem(uint64_t id,
                 std::string* pProblemData);
bool writeProblem(uint64_t id,
                  const std::string& problemData);

bool parseProblem(const std::string& problemData,
                  Problem* pProblem,
                  std::string* pMsg = nullptr);
bool parseProblem(uint64_t id,
                  Problem* pProblem,
                  std::string* pMsg = nullptr);
void formatProblem(const Problem& problem,
                   std::string* pProblemData);

bool readSolution(uint64_t id,
                  std::string* pSolutionData);
bool writeSolution(uint64_t id,
                   const std::string& solutionData);

bool parseSolution(const std::string& solutionData,
                   Solution* pSolution,
                   std::string* pMsg = nullptr);
bool parseSolution(uint64_t id,
                   Solution* pSolution,
                   std::string* pMsg = nullptr);
void formatSolution(const Solution& solution,
                    std::string* pSolutionData);

#endif
