#include "PruneProblem.hpp"
#include "Problem.hpp"
#include "Checker.hpp"

using std::vector;

bool pruneProblem(Problem& problem)
{
    if (problem.pillars.empty()) return false;

    double xs[2] = {
        problem.stageX + 5.0,
        problem.stageX + problem.stageWidth - 5.0 };
    double ys[2] = {
        problem.stageY + 5.0,
        problem.stageY + problem.stageHeight - 5.0 };

    uint64_t attendeeCount = problem.attendees.size();
    uint64_t pillarCount = problem.pillars.size();

    vector<uint8_t> removeAttendee(attendeeCount);
    vector<Pillar> newPillars;

    for (uint64_t pillarIdx = 0; pillarIdx < pillarCount; pillarIdx++)
    {
        auto& pillar = problem.pillars[pillarIdx];
        bool keepPillar = false;
        for (uint64_t attendeeIdx = 0; attendeeIdx < attendeeCount; attendeeIdx++)
        {
            if (removeAttendee[attendeeIdx]) continue;
            auto& attendee = problem.attendees[attendeeIdx];
            Coord attendeeCoord = attendee.coord;

            AttendeeChecker ac(attendeeCoord, pillar.coord, pillar.radius);
            BlockedStatus blockedStatus = ac.checkRectFull(xs, ys);
            if (blockedStatus == BlockedStatus::Blocked)
            {
                removeAttendee[attendeeIdx] = 1;
                continue;
            }
            if (blockedStatus != BlockedStatus::Unblocked)
            {
                keepPillar = true;
                break;
            }
        }
        if (keepPillar)
        {
            newPillars.push_back(pillar);
        }
    }

    vector<Attendee> newAttendees;
    for (uint64_t attendeeIdx = 0; attendeeIdx < attendeeCount; attendeeIdx++)
    {
        if (!removeAttendee[attendeeIdx])
        {
            newAttendees.push_back(problem.attendees[attendeeIdx]);
        }
    }

    bool pruned = false;
    if (newAttendees.size() != problem.attendees.size())
    {
        problem.attendees = std::move(newAttendees);
        pruned = true;
    }
    if (newPillars.size() != problem.pillars.size())
    {
        problem.pillars = std::move(newPillars);
        pruned = true;
    }
    return pruned;
}
