#include "Common.hpp"
#include "ProblemCount.hpp"
#include "Problem.hpp"
#include "Best.hpp"
#include "IsValid.hpp"
#include "CalcScore.hpp"
#include "AttendeeIndex.hpp"
#include "FastScore.hpp"
#include "FormatUtils.hpp"

using std::string;
using std::vector;

int main(int argc, char *argv[])
{
    string msg;

    for (uint64_t id = 1; id <= problemCount; id++)
    {
        printf("%2" PRIu64 "  ", id);

        Problem problem;
        if (!parseProblem(id, &problem, &msg))
        {
            printf("%s\n", msg.c_str());
            continue;
        }

        Best best;
        best.load(id);

        Solution oldSolution(best.getSolution());

        if (!isValid(problem, oldSolution))
        {
            printf("not valid\n");
            continue;
        }

        int64_t oldScore = best.getScore();

        printf("%15s  ", formatWithCommas(oldScore).c_str());

        Solution solution(oldSolution);

        AttendeeIndex attendeeIndex(problem);
        FastScore fastScore(attendeeIndex);

        fastScore.init(problem, solution);
        int64_t score = fastScore.setVolumes(problem, solution);

        if (score != calcScore(problem, solution))
        {
            printf("score mismatch\n");
            continue;
        }

        printf("%15s  ", formatWithCommas(score).c_str());

        if (score < oldScore)
        {
            printf("worse\n");
            continue;
        }

        if (score == oldScore)
        {
            printf("same\n");
            continue;
        }

        if (best.update(solution))
        {
            printf("BEST\n");
        }
        else
        {
            printf("better\n");
        }
    }

    return 0;
}
