#include "Common.hpp"
#include "Problem.hpp"
#include "ProblemCount.hpp"
#include "Best.hpp"
#include "FormatUtils.hpp"

using std::string;
using std::vector;

int main(int argc, char *argv[])
{
    string msg;

    double maxRoomWidth = 0.0;
    double maxRoomHeight = 0.0;
    double maxStageWidth = 0.0;
    double maxStageHeight = 0.0;
    uint64_t maxAttendeeCount = 0;
    uint64_t maxMusicianCount = 0;
    uint64_t maxInstrumentCount = 0;
    uint64_t maxPillarCount = 0;
    int64_t totalScore = 0;

    printf(" id   roomW   roomH  stageW  stageH   att   mus   ins   pil            score\n");
    printf("\n");
    for (uint64_t id = 1; id <= problemCount; id++)
    {
        printf("%3" PRIu64 "  ", id);

        Problem problem;
        if (!parseProblem(id, &problem, &msg))
        {
            printf("%s\n", msg.c_str());
            continue;
        }

        Best best;
        best.load(id);
        int64_t score = best.getScore();

        uint64_t attendeeCount = problem.attendees.size();
        uint64_t musicianCount = problem.musicians.size();
        uint64_t instrumentCount = problem.instruments.size();
        uint64_t pillarCount = problem.pillars.size();

        printf("%6.1f  %6.1f  %6.1f  %6.1f  %4" PRIu64 "  %4" PRIu64 "  %4" PRIu64 "  %4" PRIu64 "  %15s\n", problem.roomWidth, problem.roomHeight, problem.stageWidth, problem.stageHeight, attendeeCount, musicianCount, instrumentCount, pillarCount, formatWithCommas(score).c_str());

        maxRoomWidth = std::max(maxRoomWidth, problem.roomWidth);
        maxRoomHeight = std::max(maxRoomHeight, problem.roomHeight);
        maxStageWidth = std::max(maxStageWidth, problem.stageWidth);
        maxStageHeight = std::max(maxStageHeight, problem.stageHeight);
        maxAttendeeCount = std::max(maxAttendeeCount, attendeeCount);
        maxMusicianCount = std::max(maxMusicianCount, musicianCount);
        maxInstrumentCount = std::max(maxInstrumentCount, instrumentCount);
        maxPillarCount = std::max(maxPillarCount, pillarCount);
        totalScore += score;
    }
    printf("\n");
    printf("max  %6.1f  %6.1f  %6.1f  %6.1f  %4" PRIu64 "  %4" PRIu64 "  %4" PRIu64 "  %4" PRIu64 "  %15s\n", maxRoomWidth, maxRoomHeight, maxStageWidth, maxStageHeight, maxAttendeeCount, maxMusicianCount, maxInstrumentCount, maxPillarCount, formatWithCommas(totalScore).c_str());

    return 0;
}
