#include "Common.hpp"
#include "ProblemCount.hpp"
#include "Problem.hpp"
#include "Best.hpp"
#include "IsValid.hpp"
#include "CalcScore.hpp"
#include "StringUtils.hpp"
#include "FileUtils.hpp"
#include "FormatUtils.hpp"

using std::string;

int main(int argc, char *argv[])
{
    string msg;

    for (uint64_t id = 1; id <= problemCount; id++)
    {
        printf("%2" PRIu64 "  ", id);

        Problem problem;
        if (!parseProblem(id, &problem, &msg))
        {
            printf("%s\n", msg.c_str());
            continue;
        }

        Best best;
        best.load(id);

        int64_t oldScore = best.getScore();

        printf("%15s  ", formatWithCommas(oldScore).c_str());

        string solutionFileName = strprintf("../merge/%" PRIu64 "", id);

        string solutionData;
        if (!readFile(solutionFileName, &solutionData))
        {
            printf("Error reading %s\n", solutionFileName.c_str());
            continue;
        }

        Solution solution;
        if (!parseSolution(solutionData, &solution, &msg))
        {
            printf("%s\n", msg.c_str());
            continue;
        }

        if (!isValid(problem, solution))
        {
            printf("not valid\n");
            continue;
        }

        int64_t score = calcScore(problem, solution);

        printf("%15s  ", formatWithCommas(score).c_str());

        if (score < oldScore)
        {
            printf("worse\n");
            continue;
        }

        if (score == oldScore)
        {
            printf("same\n");
            continue;
        }

        bool saveChanges = true;
        if (saveChanges && best.update(solution))
        {
            printf("BEST\n");
        }
        else
        {
            printf("better\n");
        }
    }

    return 0;
}
