#include "Common.hpp"
#include "ProblemCount.hpp"
#include "Best.hpp"
#include "Scores.hpp"
#include "FormatUtils.hpp"

using std::string;
using std::vector;

int main(int argc, char *argv[])
{
    string msg;

    Userboard userboard;
    if (!parseUserboard(&userboard, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    vector<int64_t> myScores;
    getMyScores(userboard, &myScores);

    int64_t totalSubmittedScore = 0;
    int64_t totalScore = 0;
    int64_t totalScoreDelta = 0;

    auto printTotals = [&]()
    {
        printf("    %15s  %15s  %15s\n", formatWithCommas(totalSubmittedScore).c_str(), formatWithCommas(totalScore).c_str(), formatWithCommas(totalScoreDelta).c_str());
    };

    for (uint64_t id = 1; id <= problemCount; id++)
    {
        int64_t submittedScore = myScores[id];

        Best best;
        best.load(id);
        int64_t score = best.getScore();

        int64_t scoreDelta = score - submittedScore;

        printf("%2" PRIu64 "  %15s  %15s  %15s\n", id, formatWithCommas(submittedScore).c_str(), formatWithCommas(score).c_str(), formatWithCommas(scoreDelta).c_str());

        totalSubmittedScore += submittedScore;
        totalScore += score;
        totalScoreDelta += scoreDelta;

        if (id == 55)
        {
            printf("\n");
            //printTotals();
            //printf("\n");
        }
    }

    printf("\n");
    printTotals();

    return 0;
}
