#include "Common.hpp"
#include "FileUtils.hpp"
#include "Problem.hpp"
#include "CalcScore.hpp"

using std::string;
using std::vector;

void testScore(uint64_t testId,
               const string& problemFileName,
               const string& solutionFileName,
               bool forceUseCloseness,
               int64_t expectedScore)
{
    printf("%2" PRIu64 "  ", testId);
    fflush(stdout);

    string msg;

    string problemData;
    if (!readFile(problemFileName, &problemData))
    {
        printf("Error reading problem file: %s\n", problemFileName.c_str());
        return;
    }

    Problem problem;
    if (!parseProblem(problemData, &problem, &msg))
    {
        printf("%s\n", msg.c_str());
        return;
    }

    string solutionData;
    if (!readFile(solutionFileName, &solutionData))
    {
        printf("Error reading solution file: %s\n", solutionFileName.c_str());
        return;
    }

    Solution solution;
    if (!parseSolution(solutionData, &solution, &msg))
    {
        printf("%s\n", msg.c_str());
        return;
    }

    int64_t score =
        forceUseCloseness ?
        calcScore(problem, solution, true) :
        calcScore(problem, solution);

    if (score == expectedScore)
    {
        printf("ok    %" PRIi64 "\n", score);
    }
    else
    {
        printf("FAIL  %" PRIi64 ", expected %" PRIi64 "\n", score, expectedScore);
    }
}

int main(int argc, char *argv[])
{
    testScore(1,
              "../examples/problem1.json",
              "../examples/solution1.json",
              false,
              5343);
    testScore(2,
              "../examples/problem1-pillars.json",
              "../examples/solution1.json",
              false,
              3270);
    testScore(3,
              "../examples/problem1.json",
              "../examples/solution1.json",
              true,
              5357);

    return 0;
}
